      SUBROUTINE BT(N,X,F,G,COMPFG,FM,EPS,MAXCOM,MAXIT,RESET,
     *              IPRINT,IFAIL,IWORK,WORK,LWORK)
Cf2py intent(in) X
Cf2py intent(out) X
Cf2py intent(in) F
Cf2py intent(out) F
Cf2py intent(in) G
Cf2py intent(in) FM
Cf2py intent(in) EPS
Cf2py intent(in) MAXCOM
Cf2py intent(in) MAXIT
Cf2py intent(in) RESET
Cf2py intent(in) IPRINT
Cf2py intent(out) IFAIL
Cf2py intent(in) IWORK
Cf2py intent(in) WORK
Cf2py intent(in) LWORK
C
C     B(undle)T(rust)-Algorithm (trying to minimize a convex function)
C
C     Double Precision version
C
C     N      : dimension of the problem
C     X      : vector, dimension N
C              Input : starting point x0
C              Output: solution point
C     F      : Input : f(x0)
C              Output: f(solution)
C     G      : Input : subgradient g(x0)
C     COMPFG : SUBROUTINE, computes f(x) and a subgradient g(x),
C              Parameter: N,X,F,G
C     FM     : estimate of the minimum of f, fm < f(x0)
C     EPS    : stopping criterion,
C              eucl. norm of alpha-subgradient =< EPS and alpha =< eps
C     MAXCOM : maximal number of calls to COMPFG
C     MAXIT  : maximal number of iterations
C     RESET  : maximal number of stored subgradients, RESET > 3
C     IPRINT : to control printout
C              IPRINT=0: no printout
C              IPRINT=1: final printout
C              IPRINT=2: in iteration k
C                        niter= number of the iteration
C                        ncomp= number of calls to compfg
C                        f    = f(xk)
C                        gn   = norm of an alpha-subgradient at xk
C                      ( gives information about optimality:
C                        f(y) >= f(xk)-gn*norm(y-xk)-alpha for all y )
C              IPRINT>2: debug printout
C     IFAIL  : information about the result of BT
C              IFAIL=0: everything is okay
C              IFAIL=1: number of calls of compfg > MAXCOM
C              IFAIL=2: number of iterations > MAXIT
C              IFAIL=3: 'T too small'
C              IFAIL=4: not enough working space
C              IFAIL=5: failure in quadratic program
C     IWORK  : integer working array of dimension RESET
C     WORK   : working array of dimension LWORK
C     LWORK  : dimension of WORK,
C              LWORK  >= (RESET+N+9)*RESET+4*N+10
C
C*************************************************
C  Version 1.3 of Bundle Trust, October 20, 1989 *
C                                                *
C  Copyright by:                                 *
C                                                *
C  Helga Schramm                                 *
C  Mathematisches Institut                       *
C  Universitaet Bayreuth                         *
C  Postfach 101251                               *
C  D-8580 Bayreuth                               *
C*************************************************
C
      INTEGER          N,MAXIT,MAXCOM,IPRINT,IFAIL,LWORK,RESET
      DOUBLE PRECISION F,FM,EPS
      INTEGER           IWORK(RESET)
      DOUBLE PRECISION X(N),G(N),WORK(LWORK)
      EXTERNAL         COMPFG
      DOUBLE PRECISION TMP(1)
Cf2py  F = COMPFG(N,X,TMP,G)

C
C    local variables
C
      INTEGER          LALPHA,LXN,LA,LHESS,LY,LYA,LD,LDL,LDU,
     *                 LWORK1,LWORK2,LWORKT
C
C
      LALPHA= 1
      LXN   = LALPHA+RESET
      LA    = LXN+N
      LHESS = LA+RESET*N
      LY    = LHESS+RESET*(RESET+1)/2
      LYA   = LY+RESET
      LD    = LYA+RESET
      LDL   = LD+N
      LDU   = LDL+N
      LWORK1= LDU+N
      LWORK2= (RESET+11)*RESET/2+10
      LWORKT= LWORK1+LWORK2-1
      IFAIL = -1
      IF (LWORK .LT. LWORKT) IFAIL= 4
      IF (IPRINT .GT. 0) THEN
         WRITE(*,*)
         WRITE(*,*) 'BT-Algorithm'
         WRITE(*,*) '============'
         WRITE(*,*)
         WRITE(*,*) 'workspace provided is    WORK(',LWORK,')'
         WRITE(*,*) 'to solve problem we need WORK(',LWORKT,')'
         WRITE(*,*)
      ENDIF
      IF (IFAIL .GT. 0) THEN
         IF (IPRINT .GT. 5) THEN
            WRITE(*,*) 'Not enough workspace to solve problem'
            WRITE(*,*) 'IFAIL= ', IFAIL
         ENDIF
         RETURN
      ENDIF
      CALL BT1(N,X,F,G,COMPFG,WORK(LALPHA),FM,EPS,MAXCOM,MAXIT,
     *         RESET,IPRINT,IFAIL,WORK(LXN),WORK(LA),WORK(LHESS),
     *         IWORK,WORK(LWORK1),LWORK2,WORK(LY),WORK(LYA),
     *         WORK(LD),WORK(LDL),WORK(LDU))
      END
      SUBROUTINE BT1(N,X,F,G,COMPFG,ALPHA,FM,EPS,MAXCOM,MAXIT,RESET,
     *              IPRINT,IFAIL,XN,A,HESS,IWORK,WORK,LWORK,Y,YA,
     *              D,DL,DU)
      INTEGER          N,MAXIT,MAXCOM,IPRINT,IFAIL,LWORK,RESET
      DOUBLE PRECISION F,FM,EPS
      INTEGER          IWORK(RESET)
      DOUBLE PRECISION X(N),G(N),ALPHA(RESET),XN(N),A(RESET,N),
     *                 HESS(RESET*(RESET+1)/2),WORK(LWORK),
     *                 Y(RESET),YA(RESET),D(N),DL(N),DU(N)
      EXTERNAL         COMPFG,MLIS3
C
      DOUBLE PRECISION MU,ZERO
      PARAMETER (MU  = 1.0D-1,
     *           ZERO= 1.0D-12)
C
C    local variables
C
      INTEGER          I,J,NITER,NITER1,NN,NCOMP,INFORM,LALT,K,L,MODE,
     *                 LOGIC,IS,LIN,KK,K1,LINF,NAC
      DOUBLE PRECISION T,FN,V,VDA,VSS,ADA,VGXAL,W,VK,GD,TL,TU,SU,
     *                 ALN,ALM,ZM,ZP,ALP,GG,TP,TMIN,FPN,D2,TPS,
     *                 TOL,NU,VV,TA,TLL,TUL,ALL,ALU
      LOGICAL          KRIT,LLIN
      INTRINSIC        DABS,DSQRT
C   svnvish: Ugly hack
      DOUBLE PRECISION TMP(1)
C
C    start
C
      KRIT  = .TRUE.
      LLIN  = .FALSE.
      LINF  = 0
      NCOMP = 1
      NITER = 0
      NITER1= 0
      IFAIL = -1
      INFORM= 0
      IS= 0
      NN= 0
      ALPHA(1)= 0.0D0
      DO 1 I= 1,RESET
         YA(I)= -1.0D0
         Y(I) =  1.0D0
    1 CONTINUE
      SU = DMAX1(100.0D0*(F-FM),1.0D0)
      T  = 0.0D0
      TP = 0.0D0
      TLL= 0.0D0
      TUL= 0.0D0
      GG = 0.0D0
      DO 2 I=1,N
         GG= GG+G(I)*G(I)
    2 CONTINUE
      ZP = DSQRT(GG)
      ALP= 0.0D0
      ALM= 0.0D0
      ALN= 0.0D0
      ALL= 0.0D0
      ALU= 0.0D0
      IF (ZP .LE. EPS) THEN
         IFAIL= 0
         IF (IPRINT .GT. 1) WRITE(*,*) 'x already optimal'
         RETURN
      ENDIF
      IF (IPRINT .GT. 1) WRITE(*,'(3X,A,2X,A,8X,A,16X,A,13X,A)')
     *                        'niter','ncomp','f','gn','alpha'
      IF (IPRINT .GT. 1)
     *   WRITE(*,'(A,I5,A,I6,A,E16.8,A,E16.8,A,E16.8)')
     *           ' ',NITER1+1,' ',NCOMP,' ',F,' ',ZP,' ',ALP
C
C    iteration
C
  100 CONTINUE
      TA= T
      IF (KRIT) THEN
         IF (NITER .GE. RESET .OR. NN .EQ. RESET+100) THEN
C
C    reset
C
            IF (NN .EQ. RESET+100) THEN
               DO 3 I=1,N
                  A(1,I)= -D(I)/T
    3         CONTINUE
               ALPHA(1)= ALP
               DO 5 J=1,NITER
                  IF (ABS(ALPHA(J)) .LT. ZERO) THEN
                     ALPHA(2)= DMAX1(ALPHA(J),0.0D0)
                     DO 4 I=1,N
                        A(2,I)= A(J,I)
    4               CONTINUE
                  ENDIF
    5          CONTINUE
               NN= 2
               NITER= 2
               GOTO 111
            ENDIF
            NN= 0
            DO 7 J=1,NITER
            IF ((Y(J).GE.ZERO) .OR. (ABS(ALPHA(J)) .LT. ZERO)) THEN
               NN= NN+1
               ALPHA(NN)= DMAX1(ALPHA(J),0.0D0)
               DO 6 I=1,N
                  A(NN,I)= A(J,I)
    6          CONTINUE
            ENDIF
    7       CONTINUE
            IF (NN .GE. RESET) THEN
               NN= 2
               ALPHA(1)= ALP
               DO 8 I=1,N
                  A(1,I)= -D(I)/T
    8          CONTINUE
               DO 10 J=1,NITER
                  IF (ABS(ALPHA(J)) .LT. ZERO) THEN
                     ALPHA(2)= DMAX1(ALPHA(J),0.0D0)
                     DO 9 I=1,N
                        A(2,I)= A(J,I)
    9                CONTINUE
                  ENDIF
   10          CONTINUE
            ENDIF
  111       NITER= NN
            DO 11 I=1,RESET
               Y(I)= 0.0D0
               YA(I)= -1.0D0
   11       CONTINUE
            DO 12 I=NITER+1,RESET
               ALPHA(I)= 0.0D0
   12       CONTINUE
C
C    hess update
C
            DO 15 K=1,NITER
               LALT= K*(K-1)/2
               DO 14 J=1,K
                  K1= LALT+J
                  HESS(K1)= 0.0D0
                  DO 13 I=1,N
                     HESS(K1)= HESS(K1)+A(J,I)*A(K,I)
   13             CONTINUE
   14          CONTINUE
   15       CONTINUE
            KK= 1
            Y(1)= 1.0D0
            MODE= 1
            IF (IPRINT .GT. 1) THEN
               WRITE(*,*)
               WRITE(*,*) 'Reset, nn=',NN
               WRITE(*,*)
            ENDIF
         ENDIF
C
C    end of reset
C
         NITER1= NITER1+1
         IF (IFAIL .LT. 0 .AND. NITER1 .GE. MAXIT) THEN
            IFAIL= 2
            IF (IPRINT .GT. 1) WRITE(*,*) 'max. number of iterations'
            MAXCOM= NCOMP
         ENDIF
C
C    data for quad
C
         NITER= NITER+1
         ALPHA(NITER)= ALN
         DO 16 I=1,N
            A(NITER,I)= G(I)
   16    CONTINUE
         LALT= NITER*(NITER-1)/2
         DO 18 J=1,NITER
            K =LALT+J
            HESS(K)= 0.0D0
            DO 17 I=1,N
               HESS(K)= HESS(K)+A(J,I)*A(NITER,I)
   17       CONTINUE
   18    CONTINUE
         IF (NITER .EQ. 1) Y(1)= 1.0D0
         TL = 0.0D0
         TU = SU
         ZM = ZP
         ALM= ALP
      ENDIF
C
C    end krit
C
      IF (NITER .EQ. 1 .AND. KRIT) THEN
C
C    compute first t via line search
C
         DO 19 I=1,N
            D(I) = -G(I)
            XN(I)= X(I)
   19    CONTINUE
         FN= F
         T= 2.0D0*(F-FM)/ZP
         T= DMAX1(T,1.0D0/ZP)
         T= DMIN1(T,SU)
         TMIN= 1.0D-20
         FPN= -GG
         D2= GG
         TOL= DMAX1(0.1D0*(F-FM),EPS)
         CALL MLIS3 (COMPFG,N,XN,FN,FPN,T,TMIN,SU,D,D2,G,0.2D0,0.1D0,
     *               LOGIC,NCOMP,MAXCOM,X,TOL,TPS)
         IF (IPRINT. GE. 10) WRITE(*,'(5X,A,I4,3X,E16.8)')
     *                              'line search:',LOGIC,T
         V= -T*GG
         IF (IPRINT. GE. 5) THEN
            WRITE(*,'(48X,A,E16.8)') 't  =',T
            WRITE(*,'(48X,A,E16.8)') 'rho=',0.5*T*T*GG
         ENDIF
         IF (LOGIC .EQ. 3) THEN
C
C    null step
C
            ALN= DMAX1(TPS,0.0D0)
            DO 20 I=1,N
               X(I)= XN(I)
   20        CONTINUE
            KRIT= .TRUE.
            GOTO 100
         ENDIF
         IF (LOGIC .GE. 0 .AND. LOGIC. LE. 2) THEN
C
C    serious step
C
            DO 22 I=1,NITER
               GD= 0.0D0
               DO 21 J=1,N
                  GD= GD+T*D(J)*A(I,J)
   21          CONTINUE
               ALPHA(I)= ALPHA(I)+FN-F-GD
               ALPHA(I)= DMAX1(ALPHA(I),0.0D0)
   22       CONTINUE
            ALN= 0.0D0
            DO 23 I=1,N
               X(I)= XN(I)
   23       CONTINUE
            F= FN
            KRIT= .TRUE.
            GOTO 100
         ENDIF
         T= 0.1D0*(F-FM)/ZP
         T= DMAX1(T,1.0D0/ZP)
         T= DMIN1(T,SU)
         DO 210 I=1,N
            X(I)= XN(I)
  210    CONTINUE
      ENDIF
C
C    compute d(t)
C
      MODE= 2
      LLIN= .FALSE.
      IF (KRIT .OR. NITER .LE. 2) THEN
          MODE= 1
          LIN = 0
      ENDIF
      IF (MODE .EQ. 2) THEN
          Y(NITER)    = 1.0D0
          IWORK(NITER)= 1
      ENDIF
      IF (LIN .GT. 2 .AND. T .GT. TLL .AND. T .LT. TUL) THEN
C
C    piecewise linear
C
         LLIN= .TRUE.
         NU= (T-TLL)/(TUL-TLL)
         DO 24 I=1,N
            D(I)= DL(I)+NU*(DU(I)-DL(I))
   24    CONTINUE
         ALP= ALL+NU*(ALU-ALL)
         V= -ALPHA(1)+A(1,1)*D(1)
         DO 25 I=2,N
            V= V+A(1,I)*D(I)
   25    CONTINUE
         DO 27 J= 2,NITER
            VV= -ALPHA(J)+A(J,1)*D(1)
            DO 26 I=2,N
               VV= VV+A(J,I)*D(I)
   26       CONTINUE
            V= DMAX1(V,VV)
   27    CONTINUE
      ELSE
C
C    quadratic program
C
         CALL QUAD01(NITER,N,HESS,T,ALPHA,MODE,KK,IWORK,Y,ADA,VK,
     *               VDA,VSS,W,L,VGXAL,WORK,LWORK,INFORM)
C
         V= T*VDA
C
         DO 29 I=1,N
            D(I)= 0.0D0
            DO 28 J=1,NITER
               D(I)= D(I)-T*Y(J)*A(J,I)
   28       CONTINUE
   29    CONTINUE
C
         IF (IPRINT .GT. 10) THEN
            NAC= 0
            WRITE(*,*)
            WRITE(*,'(/A,I2,A,E23.15)') 'inform',INFORM,'   w',W
            WRITE(*,*) ' active subgradients:'
            DO 30 I=1,NITER
               IF (Y(I) .GE. ZERO) THEN
                 WRITE(*,*) I,Y(I),ALPHA(I)
                 NAC= NAC+1
               ENDIF
   30       CONTINUE
            WRITE(*,*) 'NAC=',NAC
         ENDIF
      ENDIF
C
C    compute zp,alpha
C
      ZP= 0.0D0
      DO 33 I=1,N
         ZP= ZP+D(I)*D(I)
   33 CONTINUE
      ZP= ZP/(T**2)
      IF (.NOT. LLIN) ALP= ADA
C
      IF (IPRINT .GT. 20) WRITE(*,*) 'zp2,alp  :',ZP,ALP
      IF (ZP .LE. -ZERO .OR. ALP .LE. -ZERO
     *    .OR. INFORM .GT. 0 .AND. LINF .LE. 3) THEN
         LINF= LINF+1
         KRIT= .TRUE.
         NN  = RESET+100
         IF (IPRINT .GT. 5) WRITE(*,*) 'linf= ',LINF
         IF (LINF .LE. 3) GOTO 100
      ENDIF
      IF (ZP .LE. -ZERO .OR. ALP .LE. -ZERO
     *   .OR. INFORM .GT. 0 .OR. LINF .GT. 3) THEN
         IF (IPRINT .GT. 1) WRITE(*,*) 'failure in qpdf2s'
         IF (IPRINT .GT. 1) WRITE(*,*) 'inform= ',INFORM
         IFAIL = 5
         MAXIT = NITER1
         MAXCOM= NCOMP
      ENDIF
C
      ZP= DSQRT(DABS(ZP))
C
      IF ((IPRINT .GT. 1 .AND. KRIT) .OR. IPRINT .GE. 5)
     *    WRITE(*,'(A,I5,A,I6,A,E16.8,A,E16.8,A,E16.8)')
     *         ' ',NITER1,' ',NCOMP,' ',F,' ',ZP,' ',ALP
      IF (IPRINT. GE. 5) THEN
         WRITE(*,'(48X,A,E16.8)') 't  =',T
         WRITE(*,'(48X,A,E16.8)') 'rho=',0.5D0*(T*ZP)**2
      ENDIF
C
      IF (IFAIL .LT. 0 .AND. NCOMP .GE. MAXCOM) THEN
         IFAIL= 1
         IF (IPRINT .GT. 1) WRITE(*,*) 'max. number of calls to compfg'
         MAXIT= NITER1
      ENDIF
C
C    stopping criterion
C
      IF ((DABS(ZP) .LE. EPS) .AND. (ALP .LE. EPS)) THEN
         IF (IPRINT .GE. 1) THEN
            WRITE(*,*)
            WRITE(*,*) 'convergence'
            WRITE(*,*)
         ENDIF
         IFAIL = 0
         MAXCOM= NCOMP
         MAXIT = NITER1
      ELSE
C
C    next trialpoint
C
      DO 32 I=1,N
         XN(I)= X(I)+D(I)
   32 CONTINUE
C
C    compute f(xn),g(xn)
C
      TMP(1)=FN
      CALL COMPFG(N,XN,TMP,G)
      FN=TMP(1)
      NCOMP= NCOMP+1
C
      LIN= LIN+1
      DO 31 I=1,NITER
         IF (YA(I)*Y(I) .LT. ZERO .AND. (DABS(YA(I))+Y(I)) .GT. ZERO)
     *      LIN= 0
         YA(I)= Y(I)
   31 CONTINUE
C
C    compute t
C
         KRIT= .FALSE.
C
C    test serious criterion
C
         IF (FN-F .LT. 0.1D0*V) THEN
            TL= T
            GD= 0.0D0
            DO 34 I=1,N
               GD= GD+G(I)*D(I)
   34       CONTINUE
            IF ((GD .GE. 0.2D0*V) .OR. (TU-T .LE. MU)) THEN
C
C    serious iteration
C
               KRIT= .TRUE.
               IS= MAX(1,IS+1)
               IF ((FN-F .LE. 0.7D0*V) .OR. (IS .GT. 2)) THEN
                  IF (ABS(V-FN+F).GE.ZERO)
     *               TP= -0.5D0*V*T/ABS(V-FN+F)
                  T= DMAX1(TP,5.0D0*T)
                  T= DMIN1(T,SU)
                  IS= 0
               ENDIF
C
C    update alpha
C
               DO 36 I=1,NITER
                  GD= 0.0D0
                  DO 35 J=1,N
                     GD= GD+A(I,J)*D(J)
   35             CONTINUE
                  ALPHA(I)= ALPHA(I)+FN-F-GD
                  ALPHA(I)= DMAX1(ALPHA(I),0.0D0)
   36          CONTINUE
               ALN= 0.0D0
               DO 37 I=1,N
                  X(I)= XN(I)
   37          CONTINUE
               F= FN
            ELSE
               IF (TU .EQ. SU) THEN
                  IF (ABS(V-FN+F).GE.ZERO) TP= -0.5D0*V*T/ABS(V-FN+F)
                  T= DMAX1(TP,5.0D0*T)
                  T= DMIN1(T,SU)
               ELSE
                  IF (ABS(V-FN+F).GE.ZERO) TP= -0.5D0*V*T/ABS(V-FN+F)
                  T= DMAX1(TP,TL+0.25D0*(TU-TL))
                  T= DMIN1(T,TL+0.5D0*(TU-TL))
               ENDIF
               IF (LIN .EQ. 1 .OR. LIN .EQ. 2) THEN
                  DO 38 I=1,N
                     DL(I)= D(I)
   38             CONTINUE
                  TLL= TL
                  ALL= ALP
               ENDIF
            ENDIF
         ELSE
            TU= T
            IF (TL .EQ. 0.0D0) THEN
C
C    test null criterion
C
               ALN= F-FN
               DO 39 I=1,N
                  ALN= ALN+G(I)*D(I)
  39           CONTINUE
C
               IF ((ALN. LE. DMAX1(ALP,EPS)) .OR.
     *            (DABS(F-FN). LE. 1.0D0*(ZM+ALM))) THEN
C
C    null iteration
C
                  KRIT= .TRUE.
                  ALN= DMAX1(ALN,0.0D0)
                  IS= MIN(-1,IS-1)
                  IF (IS .LT. -N) THEN
                     T= 0.5D0*T
                     IS= 0
                  ENDIF
               ELSE
                  T= TL+0.1D0*(TU-TL)
               ENDIF
            ELSE
               T= TL+0.5D0*(TU-TL)
               IF (LIN .EQ. 1 .OR. LIN .EQ. 2) THEN
                  DO 40 I=1,N
                     DU(I)= D(I)
   40             CONTINUE
               TUL= TU
               ALU= ALP
               ENDIF
            ENDIF
         ENDIF
      ENDIF
      IF (IFAIL .LT. 0 .AND. DABS(T-TA) .LT. ZERO*1.0D-3
     *    .AND. .NOT. KRIT) THEN
            IFAIL = 3
            IF (IPRINT .GT. 1) WRITE(*,*) 'T is wrong'
            MAXIT = NITER1
            MAXCOM= NCOMP
      ENDIF
      IF (IFAIL .LT. 0) GOTO 100
      END
      SUBROUTINE QUAD01(M,N,G,T,ALPHA,MODE,K,JS,Y,ADA,V,VDA,
     *                  VSS,W,L,VGXAL,WORK,LWORK,INFORM)
      INTEGER          M,N,MODE,K,L,LWORK,INFORM
      DOUBLE PRECISION T,ADA,V,VDA,VSS,W,VGXAL
      INTEGER          JS(M)
      DOUBLE PRECISION G(M*(M+1)/2),ALPHA(M),Y(M),WORK(LWORK)
C
C    local variables
C
      INTEGER          ITMAX,IDELMX,ITER
      DOUBLE PRECISION EPSTOP,EPSCON
      PARAMETER ( ITMAX  = 500,
     *            EPSTOP = 1.0D-12,
     *            EPSCON = 1.0D-12,
     *            IDELMX = 500      )
C
      CALL QUAD02(ITMAX,M,N,M,G,M*(M+1)/2,1.0D0/T,ALPHA,EPSTOP,
     *            EPSCON,IDELMX,MODE,K,JS,Y,ADA,V,VDA,VSS,
     *            W,L,VGXAL,WORK,LWORK,ITER,INFORM)
      END
      SUBROUTINE QUAD02(ITMAX,M,N,KMAX,G,LENG,U,A,EPSTOP,EPSCON,
     *                  IDELMX,MODE,K,JS,X,ADD,V,VDD,VSS,W,
     *                  L,VGXAL,WORK,LWORK,ITER,INFORM)
C
C
      INTEGER           ITMAX,M,N,KMAX,LENG,IDELMX,MODE,L,LWORK,
     *                  ITER,INFORM
      DOUBLE PRECISION  U,EPSTOP,EPSCON,ADD,V,VDD,VSS,W,VGXAL
      INTEGER           JS(KMAX)
      DOUBLE PRECISION  G(LENG),A(M),X(M),WORK(LWORK)
C
      INTEGER           LWORK1,K,KMAX1,K1,IDELET,KMAXA
      DOUBLE PRECISION  WOLD
C
C
      IF (MODE .LE. 1) THEN
         LWORK1=10+KMAX*(KMAX+11)/2
         IF (M .LT. 1 .OR. N .LT. 1 .OR. KMAX .LT. MIN0(M,N+1).OR.
     *       LENG .LT. M*(M+1)/2 .OR.
     *       EPSTOP .LT. 0.0D0 .OR. EPSCON .LE. 0.0D0 .OR.
     *       MODE .LT. 1 .OR. LWORK .LT. LWORK1)  THEN
            INFORM = 5
            RETURN
         ENDIF
         KMAX1=   KMAX
         WORK(1)= LWORK1
         WORK(2)= KMAX1
      ELSE
         LWORK1= WORK(1)+0.1D0
         IF (LWORK .LT. LWORK1) THEN
            INFORM= 5
            RETURN
         ENDIF
         KMAX1= WORK(2)+0.1D0
         K1   = WORK(3)+0.1D0
         IF (KMAX1 .LT. MIN0(M,N+1) .OR. EPSTOP .LT. 0.0D0 .OR.
     *       EPSCON .LE. 0.0D0 .OR. K .NE. K1) THEN
            INFORM= 5
            RETURN
         ENDIF
         WOLD  = WORK(4)
         IDELET= WORK(5)+0.1D0
      ENDIF
      KMAXA = MIN0(M,N+1)
      CALL QUAD03(ITMAX,M,N,KMAXA,G,LENG,U,A,EPSTOP,EPSCON,IDELMX,MODE,
     *            K,JS,X,ADD,V,VDD,VSS,W,L,VGXAL,ITER,INFORM,
     *            WORK(10),WORK(KMAX1+10),WORK(2*KMAX1+10),
     *            WORK(3*KMAX1+10),WORK(4*KMAX1+10),WORK(5*KMAX1+10),
     *            KMAXA*(KMAXA+1)/2,WOLD,IDELET)
      WORK(3)= K
      WORK(4)= WOLD
      WORK(5)= IDELET
      END
      SUBROUTINE QUAD03(ITMAX,M,N,KMAX,G,LENG,U,A,EPSTOP,EPSCON,IDELMX,
     *                  MODE,K,JS,X,ADD,V,VDD,VSS,W,L,VGXAL,ITER,
     *                  INFORM,Y,YHAT,YSS,YA,YE,R,MDIMR,WOLD,IDELET)
C
      INTEGER           ITMAX,M,N,KMAX,LENG,IDELMX,MODE,K,L,ITER,
     *                  INFORM,MDIMR,IDELET
      DOUBLE PRECISION  U,EPSTOP,EPSCON,ADD,V,VDD,VSS,W,VGXAL,
     *                  WOLD
      INTEGER           JS(KMAX)
      DOUBLE PRECISION  G(LENG),A(M),X(M),Y(KMAX),YHAT(KMAX),
     *                  YSS(KMAX),YA(KMAX),YE(KMAX),R(MDIMR)
C
      INTEGER           NINCRW,INCRW,INCRWM,KDIMR,ISETYA,ISETYE,NAUG,
     *                  NEXC,NDEL,NREF,IWOLD,JDEL,JADD,I,J,JA,LG,KG,
     *                  IA,JR,JG,IG,KR,JROT,JROTAT,ISTEP,KMIN1,KA,J1,
     *                  KREFAC
      DOUBLE PRECISION  T,ERRKT,ERRX,EPS1,EPS2,HALF,ONE,TWO,S,DELTA,
     *                  DELTAP,DELTAW,DMU,ERRORV,R1,R2,RHOSQ,VIOLAT,
     *                  WMIN,YBARL,YANEW,YENEW,DTAU,DGAMMA,DSIGMA,DNU,
     *                  DALPHA,S1,YAYE,YESQR,DABSS
C
      COMMON /QPLOG/ ERRKT,ERRX,NAUG,NEXC,NDEL
      DATA  EPS1, EPS2, HALF, ONE, TWO
     *    /0.5D0,1.0D-2,0.5D0,1.0D0,2.0D0/
C
C
      NINCRW= 3
      INCRW =-1
      INCRWM=-1
      IF (MODE .GT. 1) KDIMR= K*(K+1)/2
      ISETYA= 0
      IF (MODE .EQ. 2) ISETYA=1
      ISETYE= 0
      NAUG  = 0
      NEXC  = 0
      NDEL  = 0
      NREF  = 0
      ITER  =-1
      IWOLD = 0
      JDEL  = 0
      JADD  = 0
      T     = 0.0D0
      GOTO (10,500,100,151,500,500,500,513),MODE
   10 CONTINUE
   50 W= HALF*G(1)+U*A(1)
      L= 1
      KG=0
      DO 60 J=1,M
         KG= KG+J
         S= HALF*G(KG)+U*A(J)
         IF (W .GT. S) THEN
            W=S
            L=J
         ENDIF
   60 CONTINUE
   70 CONTINUE
      K      = 1
      KDIMR  = 1
      JS(1)  = L
      YHAT(1)= ONE
      LG     = L*(L+1)/2
      V      =-(G(LG)+U*A(L))
      IDELET = 0
      R(1)   = DSQRT(ONE+G(LG))
      YA(1)  = -A(L)/R(1)
      YE(1)  = ONE/R(1)
      JADD   = L
  100 CONTINUE
      DO 110 J=1,M
         X(J)=0.0D0
  110 CONTINUE
      ERRX= 0.0D0
      ADD= 0.0D0
      DO 120 J=1,K
         JA   = JS(J)
         X(JA)= YHAT(J)
         ERRX = ERRX+YHAT(J)
         ADD = ADD+A(JA)*YHAT(J)
  120 CONTINUE
      ERRX = ONE -ERRX
      VDD = 0.0D0
      VGXAL= 0.0D0
      ERRKT= 0.0D0
      DO 150 I=1,M
        S=0.0D0
        DO 140 J=1,K
           JA= JS(J)
           IG= MIN0(I,JA)
           JG= MAX0(I,JA)
           KG= (JG-1)*JG/2+IG
           S=  S+G(KG)*YHAT(J)
  140   CONTINUE
C       S=SUM OF G(I,J)*X(J) FOR J=1 THROUGH M.
        VDD= VDD+X(I)*S
        S   = S+U*A(I)
        IF (I .EQ. 1) VSS= -S
        VSS= DMAX1(VSS,-S)
        S= S+V
        DABSS= DABS(S)
        IF (X(I) .GT. 0.0D0) ERRKT=DMAX1(ERRKT,DABSS)
        IF (S .GE. VGXAL .AND. I .GT. 1) GOTO 150
        VGXAL=S
        L=I
  150 CONTINUE
      W   = HALF*VDD+U*ADD
      VDD=-(VDD+U*ADD)
  151 CONTINUE
      LG= L*(L+1)/2
      VIOLAT= VGXAL+EPSTOP*(ONE+G(LG))
      ERRORV= DMAX1(DABS(V-VDD),DABS(V-VSS),DABS(VDD-VSS))
     *        /(ONE+DABS(V))
      IF (IWOLD .EQ. 0) WOLD= W
      IWOLD = 1
      DELTAW= WOLD-W
      WOLD  = W
      IF (DELTAW .LE. 0.0D0) INCRW= INCRW+1
      IF(VIOLAT.GE.0.0D0) GOTO 800
      DO 160 J=1,K
         IF (L .EQ. JS(J)) GOTO 8160
  160 CONTINUE
      IF (ITER  .GE. ITMAX)  GOTO 8161
      IF (INCRW .EQ. NINCRW) GOTO 8162
      IF(INCRWM .EQ. -1) WMIN= W
      IF(W .EQ. WMIN) INCRWM= INCRWM+1
      IF(W .GE. WMIN) GOTO 170
         WMIN  = W
         INCRWM= 0
  170 CONTINUE
      IF (INCRWM .EQ. NINCRW) GOTO 8170
      DO 212 J=1,K
         JA  = JS(J)
         IG  = MIN0(L,JA)
         JG  = MAX0(L,JA)
         KG  = (JG-1)*JG/2+IG
         X(J)= G(KG)
         YSS(J)= ONE+G(KG)
  212 CONTINUE
      CALL QUAD04(R,YSS,KDIMR,K,1)
      DO 215 J=1,K
         Y(J)= YSS(J)
  215 CONTINUE
      RHOSQ= 0.0D0
      DO 220 J=1,K
         RHOSQ=RHOSQ+YSS(J)**2
  220 CONTINUE
      RHOSQ= (ONE+G(LG))-RHOSQ
      IF (RHOSQ .GT. EPSCON*(ONE+G(LG)) .AND. K .LT. KMAX) GOTO 270
      CALL QUAD04(R,YSS,KDIMR,K,0)
      YBARL= 0.0D0
      DO 230 J=1,K
         YBARL= YBARL+YSS(J)
  230 CONTINUE
      DELTA = YBARL-ONE
      DELTAP= 0.0D0
      DO 235 I=1,K
        IA= JS(I)
        S = 0.0D0
        DO 232 J=1,K
           JA= JS(J)
           IG= MIN0(IA,JA)
           JG= MAX0(IA,JA)
           KG= (JG-1)*JG/2+IG
           S=S+G(KG)*YSS(J)
  232   CONTINUE
  235 DELTAP= DELTAP+YSS(I)*S
      S= 0.0D0
      DO 240 J=1,K
         S=S+YSS(J)*X(J)
  240 CONTINUE
      DELTAW= (DELTAP+G(LG)*YBARL**2)-TWO*YBARL*S
      DELTAP= DABS((DELTAP+G(LG))-TWO*S)
      S     = RHOSQ-(DELTA**2+DELTAP)
      IF (DELTA .LT. -EPS1 .AND. K .LT. KMAX) GOTO 260
      T = 100./(ONE-EPS1)
      JR= 0
      DO 250 J=1,K
         IF (YSS(J) .LE. 0.0D0) GOTO 250
         IF (T*YSS(J) .LT. YHAT(J)) GOTO 250
         T = YHAT(J)/YSS(J)
         JR= J
  250 CONTINUE
      IF (HALF*T*DELTAW .LT. (EPS2-ONE-DELTA)*VGXAL) GOTO 400
      IF (K .EQ. KMAX .AND. JR .NE. 0) GOTO 400
      IF (K .EQ. KMAX) GOTO 8250
  260 CONTINUE
      RHOSQ= DMAX1(RHOSQ,DELTA**2+DELTAP)
      IF(RHOSQ .EQ. 0.0D0) GOTO 8250
  270 CONTINUE
      NAUG = NAUG+1
      ISTEP= 3
      YANEW=-A(L)
      YENEW= ONE
      KDIMR= KDIMR+1
      DO 310 J=1,K
         R(KDIMR)= Y(J)
         YANEW= YANEW-Y(J)*YA(J)
         YENEW= YENEW-Y(J)*YE(J)
         KDIMR= KDIMR+1
  310 CONTINUE
      R(KDIMR)= DSQRT(RHOSQ)
      K      = K+1
      JS(K)  = L
      YHAT(K)= 0.0D0
      YA(K)  = YANEW/R(KDIMR)
      YE(K)  = YENEW/R(KDIMR)
      JADD   = L
      T      = 0.0D0
      GOTO 500
  400 CONTINUE
      NEXC   = NEXC+1
      DO 410 J=1,K
         YHAT(J)= DMAX1(YHAT(J)-T*YSS(J),0.0D0)
  410 CONTINUE
      ISTEP= 4
  415 CONTINUE
      JDEL= JS(JR)
      IF (JR .EQ. K) GOTO 446
      IDELET= IDELET+1
      KMIN1 = K-1
      DO 420 J=JR,KMIN1
         YHAT(J)= YHAT(J+1)
         JS(J)=JS(J+1)
  420 CONTINUE
      IF (ISTEP .NE. 4) GOTO 422
      DO 421 J=JR,KMIN1
         X(J)=X(J+1)
  421 CONTINUE
  422 CONTINUE
      JROTAT= JR+1
      KR= JR*(JR+1)/2-1
      DO 431 JROT=JROTAT,K
         KR=     KR+JROT
         R1=     R(KR)
         R2=     R(KR+1)
         DMU=    DMAX1(DABS(R1),DABS(R2))
         DTAU=   DMU*DSQRT((R1/DMU)**2+(R2/DMU)**2)
         R(KR)=  DTAU
         DGAMMA= R1/DTAU
         DSIGMA= R2/DTAU
         DNU   = DSIGMA/(ONE+DGAMMA)
         DALPHA= DGAMMA*YA(JROT-1)+DSIGMA*YA(JROT)
         YA(JROT)  = DNU*(YA(JROT-1)+DALPHA)-YA(JROT)
         YA(JROT-1)= DALPHA
         DALPHA= DGAMMA*YE(JROT-1)+DSIGMA*YE(JROT)
         YE(JROT)= DNU*(YE(JROT-1)+DALPHA)-YE(JROT)
         YE(JROT-1)= DALPHA
         IF (JROT .EQ. K) GOTO 435
         KA= KR
         DO 430 J=JROT,KMIN1
            KA= KA+J
            DALPHA = DGAMMA*R(KA)+DSIGMA*R(KA+1)
            R(KA+1)= DNU*(R(KA)+DALPHA)-R(KA+1)
            R(KA)  = DALPHA
  430    CONTINUE
  431 CONTINUE
  435 KR= (JR-1)*JR/2+1
      KA= KR+JR
      DO 445 J=JR,KMIN1
        DO 440 J1=1,J
           R(KR)= R(KA)
           KR= KR+1
           KA= KA+1
  440   CONTINUE
        KA=KA+1
  445 CONTINUE
  446 CONTINUE
    6 KDIMR= KDIMR-K
      K    = K-1
      IF (DABS(A(JDEL)) .GT. ONE .AND. JR .LE. K) ISETYA= 1
      IF(ISTEP .EQ. 6) GOTO 500
  450 CONTINUE
      YANEW=-A(L)
      YENEW= ONE
      IF (K .GT. 0) GOTO 460
      KDIMR = 1
      IDELET= 0
      S1= 0.0D0
      GOTO 480
  460 CONTINUE
      IF(ISTEP.NE.4) GOTO 462
      DO 461 J=1,K
         YSS(J)=ONE+X(J)
  461 CONTINUE
      GOTO 472
  462 CONTINUE
      DO 471 J=1,K
         JA= JS(J)
         IG= MIN0(L,JA)
         JG= MAX0(L,JA)
         KG= (JG-1)*JG/2+IG
         YSS(J)=ONE+G(KG)
  471 CONTINUE
  472 CONTINUE
      CALL QUAD04(R,YSS,KDIMR,K,1)
      KDIMR= KDIMR+1
      S1   = 0.0D0
      DO 473 J=1,K
         S1= S1+YSS(J)**2
         R(KDIMR)= YSS(J)
         YANEW= YANEW-YSS(J)*YA(J)
         YENEW= YENEW-YSS(J)*YE(J)
         KDIMR= KDIMR+1
  473 CONTINUE
  480 R(KDIMR)= DSQRT(DMAX1((ONE+G(LG))-S1,0.0D0))
      K= K+1
      JS(K)= L
      IF (ISTEP .EQ. 4) YHAT(K)= T*(ONE+DELTA)
      IF (R(KDIMR) .EQ. 0.0D0) GOTO 8480
      YA(K)= YANEW/R(KDIMR)
      YE(K)= YENEW/R(KDIMR)
      IF(ISTEP .EQ. 7) GOTO 750
      JADD= L
  500 CONTINUE
  505 ITER= ITER+1
      IF (ISETYA .EQ. 0 .AND. ISETYE .EQ. 0) GOTO 513
      DO 510 J=1,K
         JA=JS(J)
         IF(ISETYA.EQ.1) YA(J)=-A(JA)
         IF(ISETYE.EQ.1) YE(J)=ONE
  510 CONTINUE
      IF(ISETYA .EQ. 1) CALL QUAD04(R,YA,KDIMR,K,1)
      IF(ISETYE .EQ. 1) CALL QUAD04(R,YE,KDIMR,K,1)
      ISETYA= 0
      ISETYE= 0
  513 CONTINUE
      YESQR= 0.0D0
      YAYE = 0.0D0
      DO 515 J=1,K
         YESQR= YESQR+YE(J)**2
         YAYE=YAYE+YE(J)*YA(J)
  515 CONTINUE
  516 CONTINUE
      V= (ONE-U*YAYE)/YESQR
      DO 517 J=1,K
         Y(J)=U*YA(J)+V*YE(J)
  517 CONTINUE
  540 CALL QUAD04(R,Y,KDIMR,K,0)
      V=  ONE-V
  550 CONTINUE
      JR= 0
      DO 551 J=1,K
         IF(Y(J).LE.0.0D0) JR=J
  551 CONTINUE
      JADD= 0
      IF (JR .GT. 0) GOTO 600
      DO 560 J=1,K
         YHAT(J)= Y(J)
  560 CONTINUE
      JDEL= 0
      T   = ONE
      GOTO 100
  600 CONTINUE
      NDEL= NDEL+1
      T   = ONE
      DO 610 J=1,K
        S1= YHAT(J)-Y(J)
        IF (S1 .LE. 0.0D0) GOTO 610
        S1= YHAT(J)/S1
        IF (T .LT. S1) GOTO 610
        T = S1
        JR= J
  610 CONTINUE
      DO 620 J=1,K
         YHAT(J)=DMAX1(YHAT(J)+T*(Y(J)-YHAT(J)),0.0D0)
  620 CONTINUE
      ISTEP= 6
      GOTO 415
  700 CONTINUE
      NREF  = NREF+1
      ISTEP = 7
      IDELET= 0
      KREFAC= K
      K     = 0
  750 CONTINUE
      IF (K .EQ. KREFAC) GOTO 760
      L = JS(K+1)
      LG= L*(L+1)/2
      GOTO 450
  760 CONTINUE
      ISETYA= 1
      ISETYE= 1
      JDEL  =-1000
      JADD  = 1000
      T     = 0.0D0
      GOTO 500
  800 INFORM= 0
  805 DO 810 J=1,M
         X(J)=0.0D0
  810 CONTINUE
      ERRX= 0.0D0
      DO 820 J=1,K
         JA   = JS(J)
         ERRX = ERRX+YHAT(J)
         X(JA)=YHAT(J)
  820 CONTINUE
      ERRX= ONE-ERRX
      RETURN
 8160 IF(IDELET .GT. IDELMX) GOTO 700
      INFORM= 1
      GOTO 805
 8161 INFORM= 4
      GOTO 805
 8162 INFORM= 2
      GOTO 805
 8170 INFORM= 2
      GOTO 805
 8250 IF(IDELET .GT. IDELMX) GOTO 700
      INFORM= 3
      GOTO 805
 8480 CONTINUE
      IF(IDELET .GT. 0) GOTO 700
      NDEL= NDEL+1
      IF (INCRW .EQ. NINCRW-1) INCRW=INCRW-1
      JDEL =-JS(K)
      JADD = 0
      T    = 0.0D0
      KDIMR= KDIMR-K
      K    = K-1
      S1   = 0.0D0
      DO 8484 J=1,K
         S1=S1+YHAT(J)
 8484 CONTINUE
      IF (S1 .EQ. 0.0D0) GOTO 8486
      DO 8485 J=1,K
         YHAT(J)=YHAT(J)/S1
 8485 CONTINUE
      ISETYA= 1
      ISETYE= 1
      GOTO 500
 8486 YHAT(1)= ONE
      ISETYA= 1
      ISETYE= 1
      GOTO 500
      END
      SUBROUTINE QUAD04(R,Y,MDIMR,M,ITRANS)
      INTEGER           MDIMR,M,ITRANS
      DOUBLE PRECISION  R(MDIMR),Y(M)
C
      INTEGER           I,J,K,KA,I1,IA,IOLD
      DOUBLE PRECISION  S
C
C
      IF (ITRANS.EQ.1) GOTO 3
      I= M
      K= MDIMR
      Y(I)= Y(I)/R(K)
    1 IF (I .EQ. 1) RETURN
      I1= I
      K = K-I
      KA= K
      I = I-1
      J = I
      S = Y(I)
      DO 2 IA=I1,M
        KA= KA+J
        S = S-R(KA)*Y(IA)
        J = J+1
    2 CONTINUE
      Y(I)= S/R(K)
      GOTO 1
    3 Y(1)= Y(1)/R(1)
      I= 1
      K= 1
    4 IF (I .EQ. M) RETURN
      IOLD= I
      I= I+1
      K= K+1
      S= Y(I)
      DO 5 IA=1,IOLD
        S= S-R(K)*Y(IA)
        K=K+1
    5 CONTINUE
      Y(I)= S/R(K)
      GOTO 4
      END
      SUBROUTINE MLIS3 (SIMUL,N,XN,FN,FPN,T,TMIN,TMAX,D,D2,G,
     *                  AMD,AMF,LOGIC,NAP,NAPMAX,X,TOL,TPS)
      INTEGER          N,LOGIC,NAP,NAPMAX
      DOUBLE PRECISION T,TMIN,TMAX,FN,FPN,D2,TOL,AMF,AMD,TPS
      DOUBLE PRECISION XN(N),D(N),G(N),X(N)
      EXTERNAL SIMUL
C
C     modification of MLIS2 (C. Lemarechal)
C
      INTEGER          I
      DOUBLE PRECISION TA,TG,TD,TC,TP,TESF,TESD,FPG,PS,Z1,FFN,FG,FA,FP,
     *                 FPA,FPD,SIGN,DEN,F,BARMIN,BARMUL,BARMAX,BARR,
     *                 FD,P,Z,TEST,ANUM,MC,MP
      INTRINSIC DABS,DMAX1,DMIN1
C
C
      TESF= AMF*FPN
      TESD= AMD*FPN
      TD= 0.0D0
      TG= 0.0D0
      FG= FN
      FPG= FPN
      TA= 0.0D0
      FA= FN
      FPA= FPN
      LOGIC= 0
      BARMIN= 0.010D0
      BARMUL= 3.0D0
      BARMAX= .30D0
      BARR= BARMIN
      IF (T.GT.TMIN) GO TO 20
      T= TMIN
      IF (T.LE.TMAX) GO TO 20
      TMIN= TMAX
   20 IF (FN+T*FPN.LT.FN+0.9*T*FPN) GO TO 30
      T= 2.0D0*T
      GO TO 20
   30 IF (T.GE.TMAX) THEN
          T= TMAX
          LOGIC= 1
      ENDIF
      DO 50 I=1,N
         X(I)= XN(I)+T*D(I)
   50 CONTINUE
  100 NAP= NAP+1
      IF (NAP.GT.NAPMAX) THEN
          LOGIC= 4
          IF (TG.EQ.0.) GO TO 999
          DO 120 I=1,N
             XN(I)= XN(I)+TG*D(I)
  120     CONTINUE
          CALL SIMUL (N,XN,FN,G)
          GO TO 999
      ENDIF
      CALL SIMUL(N,X,F,G)
      PS= 0.00D0
      DO 211 I=1,N
         PS= PS+G(I)*D(I)
  211 CONTINUE
      FP= PS
      FFN= F-FN
      IF(FFN.LT.T*TESF) GO TO 300
      TD= T
      FD= F
      FPD= FP
      LOGIC= 0
      IF(TG.NE.0.0D0) GO TO 500
      IF(FPD.LT.TESD) GO TO 500
      TPS= (FN-F)+TD*FPD
      P= DABS(TPS)
      IF(P.GT.TOL) GO TO 500
      LOGIC= 3
      FPN= FPD
      GO TO 999
  300 CONTINUE 
      IF(FP.LT.TESD) GO TO 320
      LOGIC= 0
      FN= F
      FPN= FP
      DO 310 I=1,N
         XN(I)= X(I)
  310 CONTINUE
      GO TO 999
  320 IF (LOGIC.EQ.0) GO TO 350
      FN= F
      FPN= FP
      DO 330 I=1,N
         XN(I)= X(I)
  330 CONTINUE
      GO TO 999
  350 TG= T
      FG= F
      FPG= FP
      TA= T
      T= 9.0D0*TG
      Z= FPN+3.0D0*FP-4.0D0*FFN/TG
      IF (Z.GT.0.0D0) T= DMIN1(T,TG*DMAX1(1.0D0,-FP/Z))
      T= TG+T
      IF(T.LT.TMAX) GO TO 900
      LOGIC= 1
      T= TMAX
      GO TO 900
  500 TEST= BARR*(TD-TG)
      PS= FP+FPA-3.0D0*(FA-F)/(TA-T)
      Z1= PS*PS-FP*FPA
      IF (Z1.GE.0.0D0) GO TO 520
      IF (FP.LT.0.0D0) TC= TD
      IF (FP.GE.0.0D0) TC= TG
      GO TO 600
  520 Z1= DSQRT(Z1)
      IF (T-TA.LT.0.) Z1= -Z1
      SIGN= (T-TA)/DABS(T-TA)
      IF ((PS+FP)*SIGN.GT.0.0D0) GO TO 550
      DEN= 2.0D0*PS+FP+FPA
      ANUM= Z1-FP-PS
      IF (DABS((T-TA)*ANUM).GE.(TD-TG)*DABS(DEN)) GO TO 530
      TC= T+ANUM*(TA-T)/DEN
      GO TO 600
  530 TC= TD
      GO TO 600
  550 TC= T+FP*(TA-T)/(PS+FP+Z1)
  600 MC= 0
      IF (TC.LT.TG) MC= -1
      IF (TC.GT.TD) MC= 1
      TC= DMAX1(TC,TG+TEST)
      TC= DMIN1(TC,TD-TEST)
      PS= FPD-FPG
      IF (PS.NE.0.D0) GO TO 620
      TP= 0.5D0*(TD+TG)
      GO TO 650
  620 TP= ((FG-FPG*TG)-(FD-FPD*TD))/PS
  650 MP= 0
      IF (TP.LT.TG) MP= -1
      IF (TP.GT.TD) MP= 1
      TP= DMAX1(TP,TG+TEST)
      TP= DMIN1(TP,TD-TEST)
      TA= T
      IF (MC.EQ.0  .AND.  MP.EQ.0) T= DMIN1(TC,TP)
      IF (MC.EQ.0  .AND.  MP.NE.0) T= TC
      IF (MC.NE.0  .AND.  MP.EQ.0) T= TP
      IF (MC.EQ.1  .AND.  MP.EQ.1) T= TD-TEST
      IF (MC.EQ.-1 .AND. MP.EQ.-1) T= TG+TEST
      IF (MC*MP.EQ.-1) T= 0.5*(TG+TD)
      IF (MC.EQ.0 .AND. MP.EQ.0) THEN
          BARR= BARMIN
        ELSE
          BARR= DMIN1(BARMUL*BARR,BARMAX)
      ENDIF
  900 FA= F
      FPA= FP
  920 IF (TD.EQ.0.0D0) GO TO 990
      IF (TD-TG.LE.TMIN) GO TO 950
      DO 930 I=1,N
         Z= XN(I)+T*D(I)
         IF (Z.NE.X(I) .AND. Z.NE.XN(I)) GO TO 990
  930 CONTINUE
  950 LOGIC= 6
      IF (TG.EQ.0.0D0) GO TO 970
      DO 960 I=1,N
         XN(I)= XN(I)+TG*D(I)
  960 CONTINUE
      CALL SIMUL(N,XN,FN,G)
  970 CONTINUE
      RETURN
  990 DO 995 I=1,N
         X(I)= XN(I)+T*D(I)
  995 CONTINUE
      GO TO 100
  999 RETURN
      END
