
// embeddings for rectangular grids with no/perimeter/full bias
// (c) 2009, 2010 Nic Schraudolph - see COPYRIGHT.txt for details

#include <iostream>
#include <vector>

#include "utils.hpp"

int main(int argc, char** argv)
{
    int i, j, m, n;

    Isinf::argv0 = argv[0];
    bool p = false;  // perimeter bias
    bool b = false;  // full bias
    bool t = false;  // toroidal

    i = 1;
    if (argc >= 2)
    {
        ++i;
        if (!strcmp(argv[1], "-b")) b = true;
        else if (!strcmp(argv[1], "-p")) p = true;
        else if (!strcmp(argv[1], "-t")) t = true;
        else --i;
    }

    if (argc - i < 1 || argc - i > 2)
    {
        std::cerr << "usage: " << argv[0]
            << " [-b | -p | -t] <height> [<width>] > <embedding>\n";
        exit(1);
    }

    if ((m = atoi(argv[i])) < 2 + t)
    {
        std::cerr << Isinf::argv0 << "height must be at least "
            << 2 + t << std::endl;
        exit(2);
    }

    if (argc - ++i <= 0) n = m;
    else if ((n = atoi(argv[i])) < 2 + t)
    {
        std::cerr << Isinf::argv0 << "width must be at least "
            << 2 + t << std::endl;
        exit(3);
    }

    bool x = false;
    bool y = false;
    bool r = false;

    /*** NOT USED (YET)

    extern int optind;
    extern char *optarg;

    int s;
    int opt, nerr = 0;
    srand(getpid());

    while ((opt = getopt(argc, argv, "xyrs:")) != EOF)
        switch (opt)
        {
            case 'x':  // shift x
                if (x || r && y) ++nerr;
                else x = true;
                break;
            case 'y':  // shift x
                if (y || r && x) ++nerr;
                else y = true;
                break;
            case 'r':  // randomize
                if (r || x && y) ++nerr;
                else r = true;
                break;
            case 's':  // random seed
                s = atoi(optarg);
                srand(s);
                break;
            default:
                ++nerr;
        }

    if (argc < optind + 1 || argc > optind + 2 || nerr)
    {
        std::cerr << "usage: " << argv[0] << 
            " [-x] [-y] [-r] [-s seed] <height> [<width>] > <embedding>\n"
            << "  -x: shift rows; -y: shift columns; "
            << "-r: randomize shifts; cannot use -xyr.\n";
        exit(1);
    }

    if ((m = atoi(argv[optind])) <= 0)
    {
        std::cerr << argv[0]
            << ": height must be positive integer" << std::endl;
        exit(2);
    }

    if (++optind == argc) n = m;
    else if ((n = atoi(argv[optind])) <= 0)
    {
        std::cerr << argv[0]
            << ": width must be positive integer" << std::endl;
        exit(3);
    }

    if (false && r)
    {
        std::cerr << argv[0]
            << ": randomisation does not work yet, sorry" << std::endl;
        exit(4);
    }

    ***/

    // regular embedding for grid
    const int bb = m*(n - 1 + t) + (m - 1 + t)*n;
    std::vector<bool> xx((m + 1)/2, x);
    std::vector<bool> yy((n + 1)/2, y);

    if (r && y)
        for (j = 0; j < (n + 1)/2; ++j)
            yy[j] = rand() & (1 << 14);

    int bx = bb - 1;
    for (i = 0; i < m; ++i)
    {
        if (r && x && !(i & 1))
            xx[i/2] = rand() & (1 << 14);

        for (j = 0; j < n; ++j)
        {
            // NE
            if (b && (i & 1) == yy[j/2] && (j & 1) != xx[i/2])
                std::cout << ' ' << ++bx;
            // N
            if (t || i > 0) std::cout << ' ' << m*(n-1+t) + (i-1+t)*n + j;
            // NW
            if (b && (i & 1) == yy[j/2] && (j & 1) == xx[i/2])
                std::cout << ' ' << ++bx;
            if (p && (i == 0 || j == 0))
                std::cout << ' ' << ++bx;
            // W
            if (t || j > 0) std::cout << ' ' << i*(n-1+t) + j + t - 1;
            // SW
            if (b && (i & 1) != yy[j/2] && (j & 1) == xx[i/2])
                std::cout << ' ' << ++bx;
            // S
            if (i < m - 1) std::cout << ' ' << m*(n-1+t) + (i+t)*n + j;
            else if (t) std::cout << ' ' << m*n + j;
            // SE
            if (b && (i & 1) != yy[j/2] && (j & 1) != xx[i/2])
                std::cout << ' ' << ++bx;
            if (p && i && j && (i == m - 1 || j == n - 1))
                std::cout << ' ' << ++bx;
            // E
            if (j < n - 1) std::cout << ' ' << i*(n-1+t) + j + t;
            else if (t) std::cout << ' ' << i*n;
            std::cout << std::endl;
        }
    }

    // bias node
    if (p)
    {
        for (i = 3; i <= m; ++i) std::cout << ' ' << bb + n + 2*(m - i);
        for (j = 0; j < n; ++j) std::cout << ' ' << bb + j;
        for (i = 2; i < m; ++i) std::cout << ' ' << bb + n + 2*i - 3;
        for (j = 0; j < n; ++j) std::cout << ' ' << bx - j;
        std::cout << std::endl;
    }
    else if (b)
    {
        for (i = 0; i < m; ++i)
        {
            for (j = 1 - xx[i/2]; j < n; ++j)
                if ((i & 1) == yy[j/2]) std::cout << ' ' << bb + i*n + j;
            for (j = n - 1; j >= 1 - xx[i/2]; --j)
                if ((i & 1) != yy[j/2]) std::cout << ' ' << bb + i*n + j;
        }
        for (i = m - 1; i >= 0; --i)
            if (!xx[i/2]) std::cout << ' ' << bb + i*n;
        std::cout << std::endl;
    }
}

