
// lbfgs.hpp - C++ wrapper for lbfgs.f Fortran code
// (c) 2008, 2009 Nic Schraudolph - see COPYRIGHT.txt for details

#include "utils.hpp"

extern "C"
{
  	void lbfgs_(const int&, const int&, double[], double&,
        const double[], const int&, double[], int[],
        const double&, const double&, double[], int&);
}

extern double compfg(const std::vector<double>&, std::vector<double>&);

double call_lbfgs(std::vector<double>& x)
{
    // customizable parameters:
    const int m = 5;           // LBFGS buffer size
    int iprint[2] = {1, 0};    // verbosity level
    const double eps = 1e-6;   // error tolerance

    // internals - do not edit below here
    const int n = x.size();
    std::vector<double> g(n);  // gradient
	  double f;                  // function value
	  int iflag = 0;             // error code
    const int diagco = 0;
	  double diag[n];
	  double W[n*(2*m + 1) + 2*m];  // workspace
    const double xtol = 1e-16; // machine precision

	  f = compfg(x, g);    // calculate initial f and g

	  do
	  {
		    lbfgs_(n, m, &x[0], f, &g[0],
            diagco, diag, iprint, eps, xtol, W, iflag);

        if (iflag < 0 || iflag > 1)
            std::cerr << Isinf::argv0 << "lbfgs: ";

		    switch (iflag)
		    {
			      case -1:
			          std::cerr << "line search failed\n";
			          exit(iflag);
			      case -2:
			          std::cerr << "non-positive element of diagonal matrix Hk0\n";
			          exit(iflag);
			      case -3:
                std::cerr << "improper input parameters (n or m not positive)\n";
			          exit(iflag);
			      case 1:
			          f = compfg(x, g);
			          break;
			      case 2:
			          std::cerr << "user must provide the diagonal matrix Hk0\n";
			          exit(iflag);
	      }
	  }
	  while (iflag != 0);

	  return f;
}

