
// margs.cpp - compute log-partition fn and edge marginal probs
// (c) 2008, 2009 Nic Schraudolph - see COPYRIGHT.txt for details

#include <fstream>
#include "superlu.hpp"

int main(int argc, char** argv)
{
    if (argc != 3)
    {
        std::cerr << "usage: " << argv[0] << " <embedding> "
            "<edgefeatures> < <params> > <probs>" << std::endl;
        exit(1);
    }
    Isinf::argv0 = argv[0];
    
    // read parameters from standard input
    std::vector<double> theta;
    std::cin >> theta;
    assert(!theta.empty());

    // read graph from first argument
    std::ifstream arg1(argv[1]);
    if (arg1.fail())
    {
        std::cerr << Isinf::argv0 << "cannot open file '" << argv[1] << "'\n";
        exit(2);
    }

    Isinf::Graph g(arg1);
    assert(g.nodes.size() > 1);
    assert(g.is_connected());
    assert(g.genus() == 0);

    // triangulate & orient graph
    size_t n = g.angulate(3);
    bool odd = g.make_odd();
    assert(odd);

    // create SuperLU interface
    Isinf::SuperLU lu(g);

    // read features from second argument
    std::ifstream arg2(argv[2]);
    if (arg2.fail())
    {
        std::cerr << Isinf::argv0 << "cannot open file '" << argv[2] << "'\n";
        exit(3);
    }

    std::vector<double> f;
    std::vector<double> e;
    double sum = 0.0;

    while (arg2.good())
    {
        while (e.size() < g.nedges - n)
        {
            arg2 >> f;
            if (e.empty() && f.empty()) exit(0);

            if (f.size() != theta.size())
            {
                std::cerr << Isinf::argv0 << "read " << f.size()
                    << " instead of " << theta.size()
                    << " features for edge " << e.size() << std::endl;
                exit(4);
            }
            sum += f*theta;
            e.push_back(exp(f*theta));
            f.clear();
        }

        // solve for log-determinant and marginals
        Isinf::CPUtimer *t = new Isinf::CPUtimer("total");
        std::cerr << lu.marginals(e)/2.0 - sum << std::endl;
        delete t;
        std::cout << e << std::endl;

        e.clear();
        sum = 0.0;
    }
}

