
// node2edge.cpp - convert node features to edge features
// (c) 2008, 2009 Nic Schraudolph - see COPYRIGHT.txt for details

#include "utils.hpp"

#include <fstream>
#include <string>
#include <map>

typedef std::pair<size_t,size_t> Edge;

int main(int argc, char** argv)
{
    std::vector<Edge> edges;
    std::vector<size_t> x;
    size_t max = 0;

    Isinf::argv0 = argv[0];
    if (argc != 2)
    {
        std::cerr << "usage: " << argv[0]
            << " <edgelist> < <nodefeatures> > <edgefeatures>" << std::endl;
        exit(1);
    }

    // read edge list from argv[1]
    std::ifstream arg1(argv[1]);
    if (arg1.fail())
    {
        std::cerr << Isinf::argv0 << "cannot open file '" << argv[1] << "'\n";
        exit(2);
    }

    arg1 >> x;
    while (x.size() >= 2)
    {
        if (x[0] == x[1])  // self-loop!
        {
            std::cerr << Isinf::argv0 << "error at node " << x[0]
                << ": self-loops not allowed" << std::endl;
            exit(3);
        }

        if (x[0] > x[1]) std::swap(x[0], x[1]);
        if (x[1] > max) max = x[1];

        edges.push_back(Edge(x[0], x[1]));
        x.clear();
        arg1 >> x;
    }
    ++max;  // number of nodes

    if (edges.size() == 0)
    {
        std::cerr << Isinf::argv0 << "no valid edges found" << std::endl;
        exit(4);
    }

    std::vector<std::vector<double> > nodefeat;
    std::vector<double> f;
    size_t numfeat = 0;

    while (std::cin.good())
    {
        // read node features from standard input
        for (size_t i = 0; i < max; ++i)
        {
            std::cin >> f;
            if (f.empty() && nodefeat.empty()) exit(0);
            if (numfeat == 0) numfeat = f.size();
            else assert(f.size() == numfeat);
            nodefeat.push_back(f);
            f.clear();
        }

        // write edge features to standard output
        for (size_t i = 0; i < edges.size(); ++i)
        {
            f.push_back(1.0);  // bias
            for (size_t j = 0; j < numfeat; ++j)
            {
                f.push_back(std::abs(nodefeat[edges[i].first][j]
                    - nodefeat[edges[i].second][j]));
            }
            std::cout << f << std::endl;
            f.clear();
        }

        nodefeat.clear();
    }
}

